package io.gitlab.jfronny.inceptum.cli;

import io.gitlab.jfronny.inceptum.Inceptum;
import io.gitlab.jfronny.inceptum.util.ArgumentTokenizer;

import java.nio.file.Files;
import java.nio.file.Path;
import java.util.LinkedHashSet;
import java.util.Set;

public class BatchCommand extends Command {
    public BatchCommand() {
        super("Executes several commands specified in a text file in one setting", "batch");
    }

    @Override
    public void invoke(InvokedCommandDescription args) {
        if (args.length == 0) {
            Inceptum.LOGGER.error("Could not start batch execution: No source file specified");
            return;
        }
        Set<InvokedCommandDescription> argsSet = new LinkedHashSet<>();
        for (String arg : args) {
            Path p = Path.of(arg);
            if (!Files.exists(p)) {
                Inceptum.LOGGER.error("Could not find " + p);
                return;
            }
            if (Files.isDirectory(p)) {
                Inceptum.LOGGER.error("Path is a directory: " + p);
                return;
            }
            try {
                for (String line : Files.readAllLines(p)) {
                    argsSet.add(new InvokedCommandDescription(ArgumentTokenizer.tokenize(line)));
                }
            } catch (Exception e) {
                Inceptum.LOGGER.error("Could not read file", e);
                return;
            }
        }
        for (InvokedCommandDescription commandDescription : argsSet) {
            commandDescription.command.invoke(commandDescription);
        }
    }

    @Override
    public boolean enableLog() {
        return true;
    }
}
