package io.gitlab.jfronny.inceptum.cli;

import java.util.*;

public class InvokedCommandDescription implements Iterable<String> {
    public static final Set<Command> COMMANDS = Set.of(new HelpCommand(), new GuiCommand(), new LaunchCommand(), new UpdateCheckCommand(), new JvmStateCommand(), new BatchCommand());
    private final List<String> args;
    public final int length;
    public final boolean wrapped;
    public final Command command;
    public InvokedCommandDescription(String[] args) throws Exception {
        if (args.length == 0) args = new String[]{"gui"};
        wrapped = args[0].equals("wrapper");
        if (wrapped && args.length == 1) args = new String[] {"wrapper", "gui"};
        {
            String cmdSel = wrapped ? args[1] : args[0];
            Command cmd = null;
            for (Command command : COMMANDS) {
                if (command.isAlias(cmdSel)) {
                    cmd = command;
                    break;
                }
            }
            if (cmd == null) {
                throw new Exception("Command not found: " + cmdSel);
            }
            command = cmd;
        }
        this.args = Arrays.asList(args).subList(wrapped ? 2 : 1, args.length);
        this.length = args.length;
    }

    public boolean contains(String param) {
        return args.contains(param.replaceAll("^[-/]*", "").toLowerCase(Locale.ROOT));
    }

    public String last() {
        return args.get(args.size() - 1);
    }

    public String get(int index) {
        return args.get(index);
    }

    public List<String> after(String param) {
        List<String> yes = null;
        for (String arg : args) {
            if (yes != null)
                yes.add(arg);
            else if (arg.equals(param.replaceAll("^[-/]*", "").toLowerCase(Locale.ROOT)))
                yes = new ArrayList<>();
        }
        return yes;
    }

    public List<String> after(int index) {
        return index + 2 < length ? args.subList(index + 1, length) : new ArrayList<>();
    }

    @Override
    public Iterator<String> iterator() {
        return args.iterator();
    }
}
