package io.gitlab.jfronny.inceptum.cli;

import io.gitlab.jfronny.inceptum.Inceptum;
import io.gitlab.jfronny.inceptum.install.Steps;
import io.gitlab.jfronny.inceptum.model.inceptum.InstanceMeta;
import io.gitlab.jfronny.inceptum.util.launch.ClientLauncher;
import io.gitlab.jfronny.inceptum.util.ProcessUtils;
import io.gitlab.jfronny.inceptum.util.Utils;
import io.gitlab.jfronny.inceptum.util.api.account.AccountManager;
import io.gitlab.jfronny.inceptum.util.launch.InstanceLauncher;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.ArrayList;
import java.util.Optional;

public class LaunchCommand extends Command {
    public LaunchCommand() {
        super("Launches the game. Optionally specify \"server\" [\"restart\"] or \"client\". Non-blocking (batch commands will continue if this is ran)", "run", "launch", "start");
    }

    @Override
    public void invoke(InvokedCommandDescription args) {
        if (args.length == 0) {
            Inceptum.LOGGER.error("You must provide an instance name or path");
            return;
        }
        int pArgIndex;
        boolean restart = false;
        boolean server = false;
        if (args.get(0).equals("server")) {
            server = true;
            if (args.length > 1 && args.get(1).equals("restart")) {
                if (args.length == 2) {
                    Inceptum.LOGGER.error("You must provide an instance name or path");
                    return;
                }
                restart = true;
                pArgIndex = 2;
            }
            else pArgIndex = 1;
        }
        else pArgIndex = args.get(0).equals("client") ? 1 : 0;
        String pArg = args.get(pArgIndex);
        Path instanceDir = Files.exists(Path.of(pArg)) ? Path.of(pArg) : Inceptum.INSTANCE_DIR.resolve(pArg);
        if (!Files.exists(instanceDir.resolve("instance.json"))) {
            Inceptum.LOGGER.error("Not a valid instance");
            return;
        }
        if (Files.exists(instanceDir.resolve("inceptum.setup.lock"))) {
            Inceptum.LOGGER.error("This instance is still being set up");
            return;
        }
        if (Files.exists(instanceDir.resolve("inceptum.lock"))) {
            try {
                if (ProcessUtils.isProcessAlive(Files.readString(instanceDir.resolve("inceptum.lock")))) {
                    Inceptum.LOGGER.error("This instance is already being ran");
                    return;
                }
            } catch (IOException e) {
                Inceptum.showError("Could not read inceptum lock", e);
                return;
            }
        }
        InstanceMeta instance;
        try {
            instance = Utils.loadObject(instanceDir.resolve("instance.json"), InstanceMeta.class);
        } catch (IOException e) {
            Inceptum.showError("Not a valid instance", e);
            return;
        }
        if (args.length > 2) {
            if (instance.arguments == null) instance.arguments = new InstanceMeta.Arguments();
            instance.arguments.client = instance.arguments.client == null ? new ArrayList<>() : new ArrayList<>(instance.arguments.client);
            instance.arguments.server = instance.arguments.server == null ? new ArrayList<>() : new ArrayList<>(instance.arguments.server);
            instance.arguments.jvm = instance.arguments.jvm == null ? new ArrayList<>() : new ArrayList<>(instance.arguments.jvm);
            instance.arguments.client.addAll(args.after(pArgIndex));
            instance.arguments.server.addAll(args.after(pArgIndex));
        }
        try {
            Steps.reDownload(instanceDir);
        } catch (IOException e) {
            e.printStackTrace();
        }
        if (server) {
            try {
                InstanceLauncher.launch(instanceDir, instance, InstanceLauncher.LaunchType.Server, restart, AccountManager.NULL_AUTH);
            } catch (InstanceLauncher.LaunchException | IOException e) {
                Inceptum.showError("Could not launch instance", e);
            }
        }
        else {
            AccountManager.loadAccounts();
            ClientLauncher.launch(instanceDir, instance);
        }
    }
}
