package io.gitlab.jfronny.inceptum.install;

import io.gitlab.jfronny.inceptum.Inceptum;
import io.gitlab.jfronny.inceptum.install.steps.*;
import io.gitlab.jfronny.inceptum.model.inceptum.InstanceMeta;
import io.gitlab.jfronny.inceptum.model.inceptum.LoaderInfo;
import io.gitlab.jfronny.inceptum.model.mojang.VersionInfo;
import io.gitlab.jfronny.inceptum.model.mojang.VersionsListInfo;
import io.gitlab.jfronny.inceptum.util.ProcessUtils;
import io.gitlab.jfronny.inceptum.util.Utils;
import io.gitlab.jfronny.inceptum.util.api.FabricMetaApi;
import io.gitlab.jfronny.inceptum.util.api.McApi;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicReference;

public class Steps {
    public static Set<Step> STEPS = new LinkedHashSet<>(List.of(
            new SetupDirsStep(),
            new DownloadJavaStep(),
            new DownloadClientStep(),
            new DownloadLibrariesStep(),
            new DownloadAssetsStep(),
            new WriteMetadataStep())
    );

    public static void reDownload() throws IOException {
        Utils.ls(Inceptum.INSTANCE_DIR, p -> {
            try {
                reDownload(p);
            } catch (IOException e) {
                Inceptum.showError("Could not execute refresh task", e);
            }
        });
    }

    public static void reDownload(Path instance) throws IOException {
        if (Files.exists(instance.resolve("inceptum.setup.lock"))) return;
        if (Files.exists(instance.resolve("inceptum.lock"))) {
            if (ProcessUtils.isProcessAlive(Files.readString(instance.resolve("inceptum.lock")))) {
                return;
            } else Files.delete(instance.resolve("inceptum.lock"));
        }
        InstanceMeta im = Utils.loadObject(instance.resolve("instance.json"), InstanceMeta.class);
        boolean found = false;
        for (VersionsListInfo version : McApi.getVersions().versions) {
            if (version.id.equals(im.getMinecraftVersion())) {
                found = true;
                VersionInfo vi = McApi.getVersionInfo(version);
                if (im.isFabric()) FabricMetaApi.addFabric(vi, im.getLoaderVersion(), FabricMetaApi.FabricVersionInfoType.Both);
                LoaderInfo li = im.isFabric()
                        ? new LoaderInfo(LoaderInfo.Type.Fabric, im.getLoaderVersion())
                        : LoaderInfo.NONE;
                SetupStepInfo info = new SetupStepInfo(vi, li, instance.getFileName().toString(), new AtomicReference<>("Reloading"));
                for (Step step : Steps.STEPS) {
                    step.execute(info, new AtomicBoolean(false));
                }
            }
        }
        if (!found) throw new IOException("Could not identify minecraft version " + im.getMinecraftVersion());
    }
}
