package io.gitlab.jfronny.inceptum.install.steps;

import io.gitlab.jfronny.inceptum.Inceptum;
import io.gitlab.jfronny.inceptum.install.SetupStepInfo;
import io.gitlab.jfronny.inceptum.install.Step;
import io.gitlab.jfronny.inceptum.model.ComparableVersion;
import io.gitlab.jfronny.inceptum.model.inceptum.InstanceMeta;
import io.gitlab.jfronny.inceptum.model.mojang.MojangFileDownload;
import io.gitlab.jfronny.inceptum.util.Utils;

import java.io.IOException;
import java.net.URISyntaxException;
import java.nio.file.FileSystem;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.concurrent.atomic.AtomicBoolean;

public class DownloadClientStep implements Step {
    @Override
    public void execute(SetupStepInfo info, AtomicBoolean stopThread) throws IOException {
        Path clientPath = Inceptum.LIBRARIES_DIR.resolve("net/minecraft/client");
        Path serverPath = Inceptum.LIBRARIES_DIR.resolve("net/minecraft/server");
        if (!Files.exists(clientPath)) Files.createDirectories(clientPath);
        if (!Files.exists(serverPath)) Files.createDirectories(serverPath);
        String minecraftVersion = InstanceMeta.getMinecraftVersion(info.version().id);
        clientPath = clientPath.resolve(minecraftVersion + ".jar");
        serverPath = serverPath.resolve(minecraftVersion + ".jar");
        if (!Files.exists(clientPath)) {
            MojangFileDownload client = info.version().downloads.client;
            info.setState("Downloading Client");
            Utils.downloadFile(client.url, client.sha1, clientPath);
        }
        Inceptum.LOGGER.info(serverPath.toString());
        Inceptum.LOGGER.info(Boolean.toString(Files.exists(serverPath)));
        if (!Files.exists(serverPath)) {
            MojangFileDownload server = info.version().downloads.server;
            if (new ComparableVersion(minecraftVersion).compareTo("1.18") >= 0) {
                info.setState("Downloading Bundler");
                Path p = Files.createTempFile("bundler", ".jar");
                Utils.downloadFile(server.url, server.sha1, p);
                try (FileSystem fs = Utils.openZipFile(p, false)) {
                    Files.copy(fs.getPath("META-INF", "versions", minecraftVersion, "server-" + minecraftVersion + ".jar"),
                            serverPath);
                } catch (URISyntaxException e) {
                    Files.delete(p);
                    throw new IOException("Could not open bundler zip", e);
                }
                Files.delete(p);
            }
            else {
                info.setState("Downloading Server");
                Utils.downloadFile(server.url, server.sha1, serverPath);
            }
        }
    }
}
