package io.gitlab.jfronny.inceptum.model.inceptum;

import io.gitlab.jfronny.inceptum.Inceptum;
import io.gitlab.jfronny.inceptum.model.curseforge.CurseforgeFingerprint;
import io.gitlab.jfronny.inceptum.model.inceptum.source.CurseforgeModSource;
import io.gitlab.jfronny.inceptum.model.inceptum.source.ModSource;
import io.gitlab.jfronny.inceptum.model.inceptum.source.ModrinthModSource;
import io.gitlab.jfronny.inceptum.util.HashUtils;
import io.gitlab.jfronny.inceptum.util.api.CurseforgeApi;
import io.gitlab.jfronny.inceptum.util.api.ModrinthApi;
import org.eclipse.jgit.annotations.Nullable;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.*;

public class ModDescription {
    public Map<ModSource, Optional<ModSource>> sources; //key: source, value: update
    public String sha1;
    public String murmur2;
    public List<String> dependents; // by file name
    public List<String> dependencies; // by file name

    public void initialize(String gameVersion) {
        boolean modrinth = false;
        boolean curseforge = false;
        for (ModSource source : sources.keySet().toArray(ModSource[]::new)) {
            if (source instanceof ModrinthModSource) modrinth = true;
            if (source instanceof CurseforgeModSource) curseforge = true;
            addSource(source, gameVersion);
        }
        if (!modrinth) {
            try {
                addSource(new ModrinthModSource(ModrinthApi.getVersionByHash(sha1).id), gameVersion);
            }
            catch (IOException e) {
                // not found
            }
        }
        if (!curseforge) {
            try {
                CurseforgeFingerprint cf = CurseforgeApi.checkFingerprint(murmur2);
                if (!cf.exactMatches.isEmpty()) {
                    CurseforgeFingerprint.Mod f = cf.exactMatches.get(0);
                    addSource(new CurseforgeModSource(f.id, f.file.id), gameVersion);
                }
            }
            catch (IOException e) {
                // not found
            }
        }
    }

    public static ModDescription of(Path mod) {
        ModDescription res = new ModDescription();
        res.sources = new LinkedHashMap<>();
        try {
            byte[] data = Files.readAllBytes(mod);
            res.sha1 = HashUtils.sha1(data);
            res.murmur2 = HashUtils.murmur2(data);
        } catch (IOException e) {
            Inceptum.LOGGER.error("Could not read file hash", e);
        }
        res.dependents = new ArrayList<>();
        res.dependencies = new ArrayList<>();
        return res;
    }

    public static ModDescription of(String sha1, String murmur2, @Nullable ModSource knownSource, String gameVersion) {
        ModDescription res = new ModDescription();
        res.sources = new LinkedHashMap<>();
        res.sha1 = sha1;
        res.murmur2 = murmur2;
        res.dependents = new ArrayList<>();
        res.dependencies = new ArrayList<>();
        if (knownSource != null) res.addSource(knownSource, gameVersion);
        res.initialize(gameVersion);
        return res;
    }

    public void addSource(ModSource source, String gameVersion) {
        try {
            sources.put(source, source.getUpdate(gameVersion));
        } catch (IOException e) {
            Inceptum.LOGGER.error("Could not check " + source.getName() + " for updates", e);
        }
    }
}
