package io.gitlab.jfronny.inceptum.model.inceptum.source;

import io.gitlab.jfronny.inceptum.model.curseforge.CurseforgeDependency;
import io.gitlab.jfronny.inceptum.model.curseforge.CurseforgeFile;
import io.gitlab.jfronny.inceptum.model.curseforge.CurseforgeMod;
import io.gitlab.jfronny.inceptum.util.HashUtils;
import io.gitlab.jfronny.inceptum.util.Utils;
import io.gitlab.jfronny.inceptum.util.api.CurseforgeApi;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.HashSet;
import java.util.Optional;
import java.util.Set;

public final class CurseforgeModSource implements ModSource {
    private final int projectId;
    private final int fileId;
    private final CurseforgeFile current;
    private final CurseforgeMod mod;

    public CurseforgeModSource(int projectId, int fileId) throws IOException {
        this.projectId = projectId;
        this.fileId = fileId;
        this.current = CurseforgeApi.getFile(projectId, fileId);
        this.mod = CurseforgeApi.getMod(projectId);
    }

    @Override
    public ModDownload download() throws IOException {
        Path path = getJarPath();
        Utils.downloadFile(current.downloadUrl, path);
        byte[] data = Files.readAllBytes(path);
        return new ModDownload(HashUtils.sha1(data), HashUtils.murmur2(data), path);
    }

    @Override
    public Set<ModSource> getDependencies() throws IOException {
        Set<ModSource> deps = new HashSet<>();
        for (CurseforgeDependency dependency : current.dependencies) {
            if (dependency.type == 3) //TODO support other types (3=required, 2=optional)
                deps.add(new CurseforgeModSource(dependency.addonId, dependency.fileId));
        }
        return deps;
    }

    @Override
    public Optional<ModSource> getUpdate(String gameVersion) throws IOException {
        //TODO test
        for (CurseforgeMod.GameVersionLatestFile file : mod.gameVersionLatestFiles) {
            if (file.gameVersion.equals(gameVersion)) {
                return file.projectFileId == fileId
                        ? Optional.empty()
                        : Optional.of(new CurseforgeModSource(projectId, file.projectFileId));
            }
        }
        return Optional.empty();
    }

    @Override
    public String getVersion() {
        return current.displayName;
    }

    @Override
    public String getName() {
        return "curseforge/" + (mod.slug == null ? mod.id : mod.slug) + '/' + current.id;
    }

    @Override
    public String getFileName() {
        return current.fileName;
    }

    @Override
    public boolean equals(ModSource other) {
        return other instanceof CurseforgeModSource cu && cu.projectId == projectId && cu.fileId == fileId;
    }

    public int getFileId() {
        return fileId;
    }

    public int getProjectId() {
        return projectId;
    }
}
