package io.gitlab.jfronny.inceptum.util.api;

import com.google.gson.reflect.TypeToken;
import io.gitlab.jfronny.inceptum.model.curseforge.CurseforgeFile;
import io.gitlab.jfronny.inceptum.model.curseforge.CurseforgeFingerprint;
import io.gitlab.jfronny.inceptum.model.curseforge.CurseforgeMod;
import io.gitlab.jfronny.inceptum.util.HttpUtils;
import io.gitlab.jfronny.inceptum.util.Utils;

import java.io.IOException;
import java.lang.reflect.Type;
import java.util.List;
import java.util.Map;

public class CurseforgeApi {
    private static final String API_URL = "https://addons-ecs.forgesvc.net/api/v2/";
    private static final Type curseforgeModListType = new TypeToken<List<CurseforgeMod>>() {}.getType();
    private static final int pageSize = 20;

    //TODO use gameVersion
    public static List<CurseforgeMod> search(String gameVersion, String query, int page, String sort) throws IOException {
        return Utils.downloadObject(Utils.buildUrl(API_URL, "addon/search", Map.of(
                "gameId", "432", // minecraft
                "modLoaderType", "4", // fabric, forge would be 1
                "sectionId", "6", // mods
                "searchFilter", query,
                "sort", sort,
                "sortDescending", "true",
                "pageSize", Integer.toString(pageSize),
                "index", Integer.toString(page * pageSize)
        )), curseforgeModListType);
    }

    public static CurseforgeMod getMod(int id) throws IOException {
        return Utils.downloadObject(API_URL + "addon/" + id, CurseforgeMod.class);
    }

    public static CurseforgeFile getFile(int modId, int fileId) throws IOException {
        return Utils.downloadObject(API_URL + "addon/" + modId + "/file/" + fileId, CurseforgeFile.class);
    }

    public static CurseforgeFingerprint checkFingerprint(String hash) throws IOException {
        return HttpUtils.post(API_URL + "fingerprint").bodyJson("[" + hash + "]").sendJson(CurseforgeFingerprint.class);
    }
}
