package io.gitlab.jfronny.inceptum.util.launch;

import io.gitlab.jfronny.inceptum.Inceptum;
import io.gitlab.jfronny.inceptum.model.inceptum.InstanceMeta;
import io.gitlab.jfronny.inceptum.util.api.account.AccountManager;
import io.gitlab.jfronny.inceptum.util.api.account.AuthInfo;

import java.io.IOException;
import java.nio.file.Path;

public class ClientLauncher {
    public static void launch(Path path, InstanceMeta instance) {
        if (AccountManager.accountMissing() && Inceptum.CONFIG.enforceAccount) {
            Inceptum.showError("You have not set up an account.\nDoing so is required to play Minecraft", "Not authenticated");
            return;
        }
        AuthInfo authInfo = AccountManager.getSelectedAccount();
        if (authInfo.equals(AccountManager.NULL_AUTH)) {
            try {
                String sysUser = System.getProperty("user.name");
                if (Inceptum.CONFIG.offlineAccountLastName == null)
                    Inceptum.CONFIG.offlineAccountLastName = sysUser;
                Inceptum.getInput("User name", Inceptum.CONFIG.offlineAccountLastName, name -> {
                    Inceptum.CONFIG.offlineAccountLastName = name.equals(sysUser) ? null : name;
                    Inceptum.saveConfig();
                    AuthInfo infoNew = new AuthInfo(name, authInfo.uuid(), authInfo.accessToken(), authInfo.userType());
                    launchI(path, instance, infoNew);
                }, () -> {});
            } catch (IOException e) {
                Inceptum.showError("Failed to request input", e);
            }
        }
        else launchI(path, instance, authInfo);
    }

    private static void launchI(Path path, InstanceMeta instance, AuthInfo authInfo) {
        try {
            InstanceLauncher.launch(path, instance, InstanceLauncher.LaunchType.Client, false, authInfo);
        } catch (InstanceLauncher.LaunchException | IOException e) {
            Inceptum.showError("Could not launch client", e);
        }
    }
}
