package io.gitlab.jfronny.inceptum.windows;

import imgui.ImGui;
import imgui.flag.ImGuiTableFlags;
import imgui.type.ImString;
import io.gitlab.jfronny.inceptum.Inceptum;
import io.gitlab.jfronny.inceptum.model.curseforge.CurseforgeMod;
import io.gitlab.jfronny.inceptum.model.inceptum.InstanceMeta;
import io.gitlab.jfronny.inceptum.model.inceptum.ModDescription;
import io.gitlab.jfronny.inceptum.model.inceptum.source.CurseforgeModSource;
import io.gitlab.jfronny.inceptum.model.inceptum.source.ModDownload;
import io.gitlab.jfronny.inceptum.model.inceptum.source.ModSource;
import io.gitlab.jfronny.inceptum.model.inceptum.source.ModrinthModSource;
import io.gitlab.jfronny.inceptum.model.modrinth.ModrinthSearchResult;
import io.gitlab.jfronny.inceptum.model.modrinth.ModrinthVersion;
import io.gitlab.jfronny.inceptum.util.ModsDirScanner;
import io.gitlab.jfronny.inceptum.util.Utils;
import io.gitlab.jfronny.inceptum.util.api.CurseforgeApi;
import io.gitlab.jfronny.inceptum.util.api.ModrinthApi;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.nio.file.Path;
import java.util.List;
import java.util.Optional;

public class AddModWindow extends Window {
    private final ImString query = new ImString("", 128);
    private final Path modsDir;
    private final InstanceMeta instance;
    private final ModsDirScanner mds;
    private int page = 0;
    private ModrinthSearchResult mr = null;
    private List<CurseforgeMod> cf = null;
    public AddModWindow(Path modsDir, InstanceMeta instance, ModsDirScanner mds) {
        super(modsDir.getParent().getFileName().toString() + " - Add Mods");
        this.modsDir = modsDir;
        this.instance = instance;
        this.mds = mds;
    }

    private void refreshMR() throws IOException {
        //TODO move to thread maybe
        mr = ModrinthApi.search(query.get(), page, instance.getMinecraftVersion());
        cf = CurseforgeApi.search(instance.getMinecraftVersion(), query.get(), page, "Popularity");
        //if (mr.hits.isEmpty()) page = 0;
    }

    @Override
    public void draw() {
        try {
            if (ImGui.inputTextWithHint("Search", "Your search query", query)) {
                refreshMR();
            }
            if (mr != null) {
                boolean hasNext = (mr.offset + mr.hits.size() < mr.limit) || cf.size() == 20;
                if (page > 0 && ImGui.button("Previous Page")) {
                    page--;
                    refreshMR();
                    if (hasNext)
                        ImGui.sameLine();
                }
                if (hasNext && ImGui.button("Next Page")) {
                    page++;
                    refreshMR();
                }
                if (ImGui.beginTabBar("ModsSelect")) {
                    if (ImGui.beginTabItem("Modrinth")) {
                        if (ImGui.beginTable("mods" + modsDir, 3, ImGuiTableFlags.SizingFixedFit | ImGuiTableFlags.Borders)) {
                            for (ModrinthSearchResult.ModResult mod : mr.hits) {
                                String modId = (mod.slug != null ? mod.slug : mod.mod_id);
                                final String idPrefix = "local-";
                                if (mod.mod_id.startsWith(idPrefix))
                                    mod.mod_id = mod.mod_id.substring(idPrefix.length());
                                //TODO detail view
                                ImGui.tableNextColumn();
                                ImGui.text(mod.title);
                                ImGui.tableNextColumn();
                                ImGui.text(mod.description);
                                ImGui.tableNextColumn();
                                boolean alreadyPresent = false;
                                for (ModsDirScanner.IWModDescription mdsMod : mds.getMods()) {
                                    alreadyPresent = mdsMod.mod().isPresent()
                                            && mdsMod.mod().get().sources.keySet().stream()
                                                .anyMatch(s -> s instanceof ModrinthModSource ms
                                                            && ms.getModId().equals(mod.mod_id));
                                    if (alreadyPresent)
                                        break;
                                }
                                if (alreadyPresent) {
                                    ImGui.text("Installed");
                                }
                                else {
                                    if (ImGui.button("Add##" + mod.mod_id)) {
                                        ModrinthVersion stable = null;
                                        ModrinthVersion beta = null;
                                        ModrinthVersion latest = null;
                                        for (ModrinthVersion version : ModrinthApi.getVersions(mod.mod_id)) {
                                            if (version.game_versions.contains(instance.getMinecraftVersion()) && version.loaders.contains("fabric")) {
                                                if (latest == null) latest = version;
                                                if (beta == null && (version.version_type == ModrinthVersion.VersionType.beta || version.version_type == ModrinthVersion.VersionType.release)) {
                                                    beta = version;
                                                }
                                                if (stable == null && (version.version_type == ModrinthVersion.VersionType.release)) {
                                                    stable = version;
                                                }
                                            }
                                        }
                                        if (stable != null) beta = stable;
                                        if (beta != null) latest = beta;
                                        if (latest == null) {
                                            Inceptum.showError("No valid version could be identified for this mod", "No version found");
                                        }
                                        else {
                                            //TODO don't block
                                            download(new ModrinthModSource(latest.id), modsDir, mds).write();
                                        }
                                    }
                                }
                                ImGui.sameLine();
                                if (ImGui.button("Web##" + mod.mod_id)) {
                                    Utils.openWebBrowser(new URI("https://modrinth.com/mod/" + modId));
                                }
                            }
                            ImGui.endTable();
                        }
                        ImGui.endTabItem();
                    }
                    if (ImGui.beginTabItem("Curseforge")) {
                        if (ImGui.beginTable("curseforge" + modsDir, 3, ImGuiTableFlags.SizingFixedFit | ImGuiTableFlags.Borders)) {
                            for (CurseforgeMod mod : cf) {
                                //TODO detail view
                                ImGui.tableNextColumn();
                                ImGui.text(mod.name);
                                ImGui.tableNextColumn();
                                ImGui.text(mod.summary);
                                ImGui.tableNextColumn();
                                boolean alreadyPresent = false;
                                for (ModsDirScanner.IWModDescription mdsMod : mds.getMods()) {
                                    alreadyPresent = mdsMod.mod().isPresent()
                                            && mdsMod.mod().get().sources.keySet().stream()
                                            .anyMatch(s -> s instanceof CurseforgeModSource ms
                                                    && ms.getProjectId() == mod.id);
                                    if (alreadyPresent)
                                        break;
                                }
                                if (alreadyPresent) {
                                    ImGui.text("Installed");
                                }
                                else {
                                    if (ImGui.button("Add##" + mod.id)) {
                                        CurseforgeMod.GameVersionLatestFile latest = null;
                                        for (CurseforgeMod.GameVersionLatestFile file : mod.gameVersionLatestFiles) {
                                            if (file.gameVersion.equals(instance.getMinecraftVersion())) {
                                                if (latest == null) latest = file;
                                            }
                                        }
                                        if (latest == null) {
                                            Inceptum.showError("No valid version could be identified for this mod", "No version found");
                                        }
                                        else {
                                            //TODO don't block
                                            download(new CurseforgeModSource(mod.id, latest.projectFileId), modsDir, mds).write();
                                        }
                                    }
                                }
                                ImGui.sameLine();
                                if (ImGui.button("Web##" + mod.id)) {
                                    Utils.openWebBrowser(new URI(mod.websiteUrl));
                                }
                            }
                            ImGui.endTable();
                        }
                        ImGui.endTabItem();
                    }
                    ImGui.endTabBar();
                }
            }
        } catch (IOException | URISyntaxException e) {
            Inceptum.LOGGER.error("Something went wrong while rendering an AddModWindow", e);
        }
    }

    public static DownloadMeta download(ModSource ms, Path modsDir, ModsDirScanner mds) throws IOException {
        for (ModsDirScanner.IWModDescription value : mds.getMods()) {
            if (value.mod().isEmpty()) continue;
            for (ModSource source : value.mod().get().sources.keySet()) {
                if (ms.equals(source)) {
                    return new DownloadMeta(new ModDownload(value.mod().get().sha1, value.mod().get().murmur2, value.path()), value.mod().get(), source, modsDir);
                }
            }
        }
        ModDownload md = ms.download();
        ModDescription manifest = ModDescription.of(md.sha1(), md.murmur2(), ms, mds.getGameVersion());
        for (ModSource dependency : ms.getDependencies()) {
            DownloadMeta depMan = download(dependency, modsDir, mds);
            depMan.description.dependents.add(md.file().getFileName().toString());
            manifest.dependencies.add(depMan.download.file().getFileName().toString());
            depMan.write();
        }
        return new DownloadMeta(md, manifest, ms, modsDir);
    }

    public static record DownloadMeta(ModDownload download, ModDescription description, ModSource source, Path modsDir) {
        public void write() throws IOException {
            Utils.writeObject(modsDir.resolve(download.file().getFileName().toString() + ".imod"), description);
        }
    }
}
